/*
 *  Engine/DB.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.IO;
using System.Xml;
using Mono.Unix;


namespace Incollector.Engine {
	
	/// <summary>Describes EntryStruct type</summary>
	public struct EntryStruct {
		
		public static string UNTAGGED = "Untagged";
		
		
		/// <summary>Entry ID</summary>
		public int id;
		
		/// <summary>Data type</summary>
		/// <remarks>EntryType of stored data</remarks>
		public string    type;
		/// <summary>Entry data</summary>
		/// <remarks>Type of data is named as type field</remarks>
		public object    data;
		
		
		public DateTime   date;
		public int        rating;
		public ArrayList  tags;
		public string     comment;
		public bool       imported;
		public bool       accepted;
		public int        version;
		public string     imid; // import id
		
		
		
		/// <summary>Returns date in selected format</summary>
		/// <param name="DateFirstYear">If true, data format is year-month-day</param>
		public string GetDate (bool DateFirstYear)
		{
			if (DateFirstYear == true)
				return date.ToString ("yyyy-MM-dd");
			else
				return date.ToString ("dd-MM-yyyy");
		}
		
		
		/// <summary>Adds a new tag to entry</summary>
		/// <param name="tag">Tag name</param>
		/// <returns>If tag exists, returns false</returns>
		public bool AddTag (string tag)
		{
			foreach (string str in tags)
				if (str == tag)
					return false;
			
			tags.Add (tag);
			RemoveTag (Catalog.GetString (UNTAGGED));
			
			return true;
		}
		
		
		/// <summary>Removes tag</summary>
		/// <param name="tag">Tag name</param>
		public void RemoveTag (string tag)
		{
			int i = 0;
			foreach (string str in tags) {
				if (str == tag) {
					tags.RemoveAt (i);
					
					if (tags.Count == 0)
						tags.Add (Catalog.GetString (UNTAGGED));
					
					return;
				}
				
				i++;
			}
			
			return;
		}
		
		
		/// <summary>If tag exists, returns true</summary>
		/// <param name="tag">Tag name</param>
		public bool TagExists (string tag)
		{
			foreach (string str in tags)
				if (str == tag)
					return true;
			
			return false;
		}
		
		
		/// <summary>Changes tag to new tag</summary>
		/// <param name="OldTag">Tag name which exists in tags list</param>
		/// <param name="NewTag">OldTag is replaced by this tag</param>
		public void ChangeTag (string OldTag, string NewTag)
		{
			for (int i = 0; i < tags.Count; i++)
				if ((string) tags [i] == OldTag) {
					tags [i] = NewTag;
					return;
				}
		}
		
		
		/// <summary>Removes all the tags.</summary>
		public void RemoveAllTags ()
		{
			tags.Clear ();
			tags.Add (Catalog.GetString (UNTAGGED));
		}
		
		
		/// <summary>Returns tags list (separated by ', ')</summary>
		public string GetTagsString (bool ShowUntagged)
		{
			if (ShowUntagged == false)
				if (tags.Count == 1)
					if ((string) tags [0] == Catalog.GetString (UNTAGGED)) // if untagged, return empty string
						return "";
			
			string rtn = "";
			foreach (string str in tags)
				if (rtn.Length > 0)
					rtn += ", " + str;
				else
					rtn += str;
			
			return rtn;
		}
		
		
		/// <summary>Returns tags list (separated by ', ')</summary>
		public string GetTagsString ()
		{
			return GetTagsString (true);
		}
		
		
		/// <summary>Returns tags list (separated by ',')</summary>
		/// <remarks>Used only by Incollector.Engine.DB"</remarks>
		public string GetTagsDBString ()
		{
			if (tags.Count == 1)
				if ((string) tags [0] == Catalog.GetString (UNTAGGED)) // if untagged, return empty string
					return "";
			
			string rtn = "";
			foreach (string str in tags)
				if (rtn.Length > 0)
					rtn += "," + str;
				else
					rtn += str;
			
			return rtn;
		}
	}
	
	
	/// <summary>Entries database</summary>
	public class DB {
		public delegate void LoadHandler   (bool LockGUI);
		public delegate void SaveHandler   (bool LockGUI);
		public delegate void AddHandler    (Engine.EntryStruct EntryStruct, bool LockGUI);
		public delegate void UpdateHandler (Engine.EntryStruct EntryStruct, Engine.EntryStruct OldEntry, bool LockGUI);
		public delegate void RemoveHandler (Engine.EntryStruct EntryStruct, bool LockGUI);
		
		public static LoadHandler   Loaded;
		public static LoadHandler   Saved;
		public static AddHandler    Added;
		public static UpdateHandler Updated;
		public static RemoveHandler Removed;
		
		
		/// <summary>Entries list</summary>
		private static ArrayList data;
		private static int       LastID;
		
		
		public static ArrayList Data {
			get { return data; }
		}
		
		
		/// <summary>Number of entries in the database.</summary>
		public static int NEntries {
			get { return data.Count; }
		}
		
		
		/// <summary>Loads database</summary>
		public static bool Load ()
		{
			LastID = 0;
			data = new ArrayList ();
			
			StreamReader reader;
			try {
				reader = new StreamReader (FilesPaths.DBFile, System.Text.Encoding.UTF8);
			} catch  {
				return false;
			}
			
			
			XmlTextReader read = new XmlTextReader (reader);
			EntryStruct    ent = new EntryStruct ();
			DataTypes.DataType dtp;
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.Element)) {
							ent = new EntryStruct ();
							
							try {
								ent.id = Convert.ToInt16 (read.GetAttribute ("id"));
								if (ent.id == 0) {
									LastID++;
									ent.id = LastID;
								} else {
									if (ent.id > LastID)
										LastID = ent.id;
								}
							} catch {
								LastID++;
								ent.id = LastID;
							}
							
							
							try {
								string [] arr  = (read.GetAttribute ("date") as string).Split ('-');
								ent.date = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
							} catch {
								ent.date = DateTime.Today;
							}
							
							
							try {
								ent.rating = Convert.ToInt16 (read.GetAttribute ("rating"));
							} catch {
								ent.rating = 0;
							}
							
							try {
								ent.version = Convert.ToInt16 (read.GetAttribute ("version"));
								if (ent.version == 0)
									ent.version = 1;
							} catch {
								ent.version = 1;
							}
							
							
							ent.imported = read.GetAttribute ("imported") == "True";
							ent.accepted = read.GetAttribute ("accepted") == "True";
							
							ent.tags = new ArrayList ();
							foreach (string tg in read.GetAttribute ("tags").Split (','))
								if (tg != "")
									ent.tags.Add (Text.PrepareText2 (tg));
							
							if (ent.tags.Count == 0)
								ent.tags.Add (Catalog.GetString (Engine.EntryStruct.UNTAGGED));
							
							ent.comment = "";
							
							continue;
						}
						
						if (read.Name == "comment") {
							ent.comment = read.ReadString ();
							continue;
						}
						
						
						dtp = DataTypes.DataTypes.GetDataType (read.Name);
						if (dtp != null)
							ent = dtp.ReadXml (read, ent);

						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.EndElement)) {
							if (ent.data != null)
								data.Add (ent);
							continue;
						}
					}
				}
			} catch {
				reader.Close ();
				return false;
			}
			
			reader.Close ();
			
			if (Loaded != null)
				Loaded (true);
			
			return true;
		}
		
		
		/// <summary>Saves database</summary>
		public static bool Save (bool Autosave)
		{
			try {
				XmlTextWriter f;
				if (Autosave == true)
					f = new XmlTextWriter (FilesPaths.DBAutosaveFile, System.Text.Encoding.UTF8);
				else
					f = new XmlTextWriter (FilesPaths.DBFile, System.Text.Encoding.UTF8);
				
				DataTypes.DataType dtp;
	
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "data", "");
					foreach (Engine.EntryStruct ent in data) {
						f.WriteStartElement ("entry");
						f.WriteAttributeString ("id", ent.id.ToString ());
						f.WriteAttributeString ("imid", ent.imid);
						f.WriteAttributeString ("date", ent.date.ToString ("yyyy-MM-dd"));
						f.WriteAttributeString ("rating", ent.rating.ToString ());
						f.WriteAttributeString ("version", ent.version.ToString ());
						f.WriteAttributeString ("imported", ent.imported.ToString ());
						f.WriteAttributeString ("accepted", ent.accepted.ToString ());
						f.WriteAttributeString ("tags", ent.GetTagsDBString ());
							f.WriteStartElement ("comment");
								f.WriteString (ent.comment);
							f.WriteEndElement ();
							
							dtp = DataTypes.DataTypes.GetDataType (ent.type);
							if (dtp != null)
								dtp.WriteXml (f, ent);
						
						f.WriteEndElement ();
					}
				f.WriteEndElement ();
	
				f.Close ();
			} catch {
			}
			
			if (Saved != null)
				Saved (true);
			
			return true;
		}
		
		
		/// <summary>Imports entries from external XML file</summary>
		public static bool ImportEntries (string FileName, bool ImportDates,
									 		bool ImportRating, bool ImportTags)
		{
			StreamReader reader;
			try {
				reader = new StreamReader (FileName, System.Text.Encoding.UTF8);
			} catch  {
				return false;
			}
			
			
			XmlTextReader read = new XmlTextReader (reader);
			EntryStruct    ent = new EntryStruct ();
			ArrayList      buf = new ArrayList ();
			DataTypes.DataType dtp;			
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.Element)) {
							LastID++;
							
							ent    = new EntryStruct ();
							ent.id = LastID;
							
							if (ImportDates) {
								try {
									string [] arr  = (read.GetAttribute ("date") as string).Split ('-');
									ent.date = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
								} catch {
									ent.date = DateTime.Today;
								}
							} else
								ent.date = DateTime.Today;
							
							if (ImportRating) {
								try {
									ent.rating = Convert.ToInt16 (read.GetAttribute ("rating"));
								} catch {
									ent.rating = 0;
								}
							} else
								ent.rating = 0;
							
							
							try {
								ent.version = Convert.ToInt16 (read.GetAttribute ("version"));
								if (ent.version == 0)
									ent.version = 1;
							} catch {
								ent.version = 1;
							}
							
							
							ent.imid = read.GetAttribute ("imid");
							ent.imported = true;
							ent.accepted = false;
							
							ent.tags = new ArrayList ();
							ent.tags.Add (Catalog.GetString (Engine.EntryStruct.UNTAGGED));

							if (ImportTags)
								foreach (string tg in read.GetAttribute ("tags").Split (','))
									if (tg != "")
										ent.AddTag (Engine.Text.PrepareText2 (tg));
							
							ent.comment = "";
							
							continue;
						}
						
						if (read.Name == "comment") {
							ent.comment = read.ReadString ();
							continue;
						}
						
						
						dtp = DataTypes.DataTypes.GetDataType (read.Name);
						if (dtp != null)
							ent = dtp.ReadXml (read, ent);

						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.EndElement)) {
							if (ent.data != null)
								buf.Add (ent);
							continue;
						}
					}
				}
			} catch {
				reader.Close ();
				return false;
			}
			
			
			foreach (Engine.EntryStruct e in buf) {
				data.Add (e);
				if (Added != null)
					Added (e, false);
			}
			
			reader.Close ();
			return true;
		}


		/// <summary>Exports selected or all the entries to external XML file</summary>
		/// <remarks>If EntriesList is empty, exports all the entries</remarks>
		public static void ExportEntries (ArrayList EntriesList, string FileName)
		{
			bool AllEntries = EntriesList.Count == 0;


			try {
				XmlTextWriter f = new XmlTextWriter
					(FileName, System.Text.Encoding.UTF8);
				DataTypes.DataType dtp;
				
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "IED", "");
					f.WriteAttributeString ("date", DateTime.Today.ToString ("yyyy-MM-dd"));
					
					foreach (Engine.EntryStruct ent in data) {
						if (AllEntries == false) {
							foreach (int id in EntriesList)
								if (ent.id == id) {
									EntriesList.Remove (ent);
									goto next;
								}
	
							continue;
						}
						
						next:
						f.WriteStartElement ("entry");
						f.WriteAttributeString ("id", ent.id.ToString ());
						f.WriteAttributeString ("imid", ent.imid);
						f.WriteAttributeString ("date", ent.date.ToString ("yyyy-MM-dd"));
						f.WriteAttributeString ("rating", ent.rating.ToString ());
						f.WriteAttributeString ("version", ent.version.ToString ());
						f.WriteAttributeString ("tags", ent.GetTagsDBString ());
							f.WriteStartElement ("comment");
								f.WriteString (ent.comment);
							f.WriteEndElement ();
							
							dtp = DataTypes.DataTypes.GetDataType (ent.type);
							if (dtp != null)
								dtp.WriteXml (f, ent);
						
						f.WriteEndElement ();
					}
				f.WriteEndElement ();
	
				f.Close ();
			} catch {
			}
		}
		
		
		/// <summary>Accepts speciefied entries.</summary>
		/// <param name="entries">Entries to accept list.</param>
		public static void AcceptEntries (ArrayList entries)
		{
			for (int i = 0; i < data.Count; i++) {
				foreach (int id in entries) {
					if (((Engine.EntryStruct) data [i]).id == id) {
						EntryStruct NewEntry = (Engine.EntryStruct) data [i];
						NewEntry.tags  = new ArrayList (((Engine.EntryStruct) data [i]).tags);
						NewEntry.version++;
						NewEntry.accepted = true;
						
						data [i] = NewEntry;
						
						if (Updated != null)
							Updated ((Engine.EntryStruct) data [i], NewEntry, false);
						
						entries.Remove (id);
						break;
					}
				}
				
				if (entries.Count == 0)
					break;
			}
		}
		
		
		/// <summary>Adds a new entry to the database</summary>
		/// <returns>Returns added entry</returns>
		/// <param name="ent">Entry to add</param>
		public static Engine.EntryStruct AddEntry (Engine.EntryStruct ent)
		{
			LastID++;
			ent.id = LastID;
			
			ent.version = 1;
			
			data.Add (ent);
			if (Added != null)
				Added (ent, true);
			
			return ent;
		}
		
		
		/// <summary>Returns entry</summary>
		/// <param name="id">ID of entry to return</param>
		public static Engine.EntryStruct GetEntryByID (int id)
		{
			foreach (Engine.EntryStruct ent in data)
				if (ent.id == id)
					return ent;
			
			return new EntryStruct ();
		}
		
		
		/// <summary>Removes entry</summary>
		/// <param name="id">ID of entry to remove</param>
		public static void RemoveEntry (Engine.EntryStruct entry)
		{
			foreach (Engine.EntryStruct ent in data)
				if (ent.id == entry.id) {
					if (Removed != null)
						Removed (entry, true);
					
					data.Remove (ent);
					return;
				}
		}
		
		
		/// <summary>Changes entry</summary>
		/// <returns>True - if operation successfully</returns>
		/// <param name="NewEntry">Entry with the same id will be replaced by itself</param>
		public static bool ChangeEntry (Engine.EntryStruct NewEntry)
		{
			int i = 0;
			foreach (Engine.EntryStruct ent in data) {
				if (ent.id == NewEntry.id) {
					EntryStruct OldEntry = (EntryStruct) data [i];
					
					NewEntry.version++;
					data [i] = NewEntry;
					
					if (Updated != null)
						Updated (NewEntry, OldEntry, true);
					
					return true;
				}
				
				i++;
			}
			
			return false;
		}
	}
}
