/*
 *  GUI/EntriesList/ListView.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using Gdk;
using Incollector.GUI;
using Incollector.Engine;


namespace Incollector.GUI {

	public class ListView : EntriesList {
		private enum ColsEnum {
			Item     = 0,
			Stars    = 1,
			ID       = 2,
			Category = 3,
			Date     = 4,
			Rating   = 5
		};
		
		
		private TargetEntry [] DestTable = new TargetEntry [] {
			new TargetEntry ("STRING", 0, 0),
			new TargetEntry ("text/uri-list", 0, 0),
			new TargetEntry ("text/plain", 0, 0),
			new TargetEntry ("_NETSCAPE_URL", 0, 0),
			new TargetEntry ("application/x-rootwin-drop", 0, 0)
		};
		
		
		private TreeView   list;
		private Gdk.Pixbuf Stars1, Stars2, Stars3, EmptyP;
		private SortByEnum SortByValue;
		private bool        ShowingListMenu;
		
		
		public override SortByEnum SortBy {
			get {
				return SortByValue;
			}
			
			set {
				SortByValue = value;
				
				switch (SortByValue) {
					case SortByEnum.Category:
						(list.Model as TreeStore).SetSortFunc (0, CompareCategory);
						(list.Model as TreeStore).SetSortColumnId (0, SortType.Ascending);
					break;
					case SortByEnum.Date:
						(list.Model as TreeStore).SetSortFunc (1, CompareDate);
						(list.Model as TreeStore).SetSortColumnId (1, SortType.Descending);
					break;
					case SortByEnum.Rating:
						(list.Model as TreeStore).SetSortFunc (2, CompareRating);
						(list.Model as TreeStore).SetSortColumnId (2, SortType.Descending);
					break;
				}
			}
		}
		
		
		/// <summary>Number of entries on the list.</summary>
		public override int NEntries {
			get { return (list.Model as TreeStore).IterNChildren (); }
		}
		
		
		/// <summary>Number of selected entries on the list.</summary>
		public override int NSelectedEntries {
			get { return GetSelectedEntries ().Count; }
		}
		
		
		/// <summary>Gets list widget as Gtk.Widget</summary>
		public override Gtk.Widget ListWidget {
			get { return list as Gtk.Widget; }
		}
		
		
		/// <summary>Creates list view widget.</summary>
		public override void Create ()
		{
			// Loading stars pixmaps
			Stars1 = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars-1.png"));
			Stars2 = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars-2.png"));
			Stars3 = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars-3.png"));
			EmptyP = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars-0.png"));
			
			
			// creating treeview widget
			list = new TreeView ();
			
			Gtk.TreeStore store = new Gtk.TreeStore (typeof (string), typeof (Gdk.Pixbuf), typeof (int), typeof (string), typeof (string), typeof (int));
			list.Model = store;
			list.EnableSearch = false;
			list.RulesHint = true;
			list.HeadersVisible = false;
			list.Selection.Mode = SelectionMode.Multiple;
			list.Selection.Changed += new EventHandler (on_selection_changed);
			list.RowActivated += new RowActivatedHandler (on_row_activated);
			list.DragDataReceived += new DragDataReceivedHandler (on_drag_data_received);
			list.DragMotion += new DragMotionHandler (on_drag_motion);
			list.ButtonReleaseEvent += new ButtonReleaseEventHandler (on_button_release_event);
			
			Gtk.Drag.DestSet (list, DestDefaults.All, DestTable, DragAction.Copy | DragAction.Move);
			
			CellRendererText txtrend = new CellRendererText ();
			txtrend.Ellipsize = Pango.EllipsizeMode.End;
			TreeViewColumn col = new TreeViewColumn ("", txtrend, "markup", 0);
			col.Expand = true;
			list.AppendColumn (col);
			
			CellRendererPixbuf rend = new CellRendererPixbuf ();
			rend.Xalign = 0;
			list.AppendColumn ("", rend, "pixbuf", 1);
			
			
			// creating scrollbar widget
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.None; 
			scroll.Add (list);
			
			this.Add (scroll);
		}
		
		
		protected void on_selection_changed (object o, EventArgs args)
		{
			SelectionChanged (list.Selection.CountSelectedRows ());
		}
		
		
		protected void on_row_activated (object o, RowActivatedArgs args)
		{
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter itr;
				(list.Model as TreeStore).GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				if ((list.Model as TreeStore).IterHasChild (itr)) {
					if (list.ExpandRow ((list.Model as TreeStore).GetPath (itr), false) == false)
						list.CollapseRow ((list.Model as TreeStore).GetPath (itr));
					
					return;
				}
				
				
				if (EntryActivated != null)
					EntryActivated (Engine.DB.GetEntryByID ((int) (list.Model
									as TreeStore).GetValue (itr, (int) ColsEnum.ID)));
			}
		}
		
		
		protected void on_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.Button == 3) && (list.Selection.CountSelectedRows () == 1)) {
				Gtk.TreeIter  itr;
				Gtk.TreeStore store = list.Model as TreeStore;
				Glade.XML     m     = Glade.XML.FromAssembly ("gui.glade", "EntriesListMenu", null);
				
				m.Autoconnect (this);
				Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("EntriesListMenu");
				
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				if (store.IterHasChild (itr) == true) {
					(m.GetWidget ("remove_entry") as Gtk.Widget).Hide ();
					(m.GetWidget ("edit_entry")   as Gtk.Widget).Hide ();
					(m.GetWidget ("separator")    as Gtk.Widget).Hide ();
					(m.GetWidget ("no_rating")    as Gtk.Widget).Hide ();
					(m.GetWidget ("rating_1")     as Gtk.Widget).Hide ();
					(m.GetWidget ("rating_2")     as Gtk.Widget).Hide ();
					(m.GetWidget ("rating_3")     as Gtk.Widget).Hide ();
				} else {
					(m.GetWidget ("remove_entries") as Gtk.Widget).Hide ();
					
					ShowingListMenu = true;
					
					Engine.EntryStruct ent = DB.GetEntryByID ((int) store.GetValue (itr, (int) ColsEnum.ID));
					switch (ent.rating) {
						case 0: (m.GetWidget ("no_rating") as Gtk.RadioMenuItem).Active = true; break;
						case 1: (m.GetWidget ("rating_1") as Gtk.RadioMenuItem).Active  = true; break;
						case 2: (m.GetWidget ("rating_2") as Gtk.RadioMenuItem).Active  = true; break;
						case 3: (m.GetWidget ("rating_3") as Gtk.RadioMenuItem).Active  = true; break;
					}
					
					ShowingListMenu = false;
				}
				
				menu.Show ();
				menu.Popup ();
			}
		}
		
		
		protected void on_add_activate (object o, EventArgs args)
		{
			if (AddEntryActivated != null)
				AddEntryActivated (o, args);
		}
		
		
		protected void on_remove_activate (object o, EventArgs args)
		{
			if (RemoveEntryActivated != null)
				RemoveEntryActivated (o, args);
		}
		
		
		protected void on_remove_entries_activate (object o, EventArgs args)
		{
			if (RemoveEntryActivated != null)
				RemoveEntryActivated (o, args);
		}
		
		
		protected void on_edit_entry_activate (object o, EventArgs args)
		{
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter itr;
				(list.Model as TreeStore).GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				if ((list.Model as TreeStore).IterHasChild (itr)) {
					if (list.ExpandRow ((list.Model as TreeStore).GetPath (itr), false) == false)
						list.CollapseRow ((list.Model as TreeStore).GetPath (itr));
					
					return;
				}
				
				
				if (EntryActivated != null)
					EntryActivated (Engine.DB.GetEntryByID ((int) (list.Model
									as TreeStore).GetValue (itr, (int) ColsEnum.ID)));
			}
		}
		
		
		protected void on_no_rating_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.TreeStore store = list.Model as TreeStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 0;
				ent.Save ();
				
			}
		}
		
		
		protected void on_rating_1_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.TreeStore store = list.Model as TreeStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 1;
				ent.Save ();
			}
		}
		
		
		protected void on_rating_2_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.TreeStore store = list.Model as TreeStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 2;
				ent.Save ();
			}
		}
		
		
		protected void on_rating_3_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.TreeStore store = list.Model as TreeStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 3;
				ent.Save ();
			}
		}
		
		
		protected void on_expand_all_activate (object o, EventArgs args)
		{
			list.ExpandAll ();
		}
		
		
		protected void on_collapse_all_activate (object o, EventArgs args)
		{
			list.CollapseAll ();
		}
		
		
		protected void on_drag_data_received (object o, DragDataReceivedArgs args)
		{
			if (DragDataReceived != null)
				DragDataReceived (o, args);
		}
		
		
		protected void on_drag_motion (object o, DragMotionArgs args)
		{
			TreePath path;
			TreeViewDropPosition pos;
			
			if (list.GetDestRowAtPos (args.X, args.Y, out path, out pos))
				list.SetDragDestRow (path, TreeViewDropPosition.IntoOrAfter);
		}
		
	
		/// <summary>Compares 2 entries by their category.</summary>
		private int CompareCategory (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			string a = (string) store.GetValue (iter1, (int) ColsEnum.Category);
			string b = (string) store.GetValue (iter2, (int) ColsEnum.Category);
			return String.Compare (a, b);
		}
		
		
		/// <summary>Compares 2 entries by their date.</summary>
		private int CompareDate (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			DateTime date1 = new DateTime (), date2 = new DateTime ();
			
			try {
				string [] arr = (store.GetValue (iter1, (int) ColsEnum.Date) as string).Split ('-');
				date1 = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
			} catch {
				date1 = DateTime.Today;
			}
			
			try {
				string [] arr = (store.GetValue (iter2, (int) ColsEnum.Date) as string).Split ('-');
				date2 = new DateTime (Convert.ToInt32 (arr [0]), Convert.ToInt32 (arr [1]), Convert.ToInt32 (arr [2]));
			} catch {
				date2 = DateTime.Today;
			}
			
			return DateTime.Compare (date1, date2);
		}
		
		
		/// <summary>Compares 2 entries by their rating.</summary>
		private int CompareRating (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			return Decimal.Compare ((int) store.GetValue (iter1, (int) ColsEnum.Rating),
								   (int) store.GetValue (iter2, (int) ColsEnum.Rating));
		}
		
		
		/// <summary>Clears entries list.</summary>
		public override void Clear ()
		{
			(list.Model as TreeStore).Clear ();
		}
		
		
		/// <summary>Expand all.</summary>
		public override void ExpandAll ()
		{
			list.ExpandAll ();
		}
		
		
		/// <summary>Sets text zoom</summary>
		public override void Zoom (int size)
		{
			list.PangoContext.FontDescription.Size = (int) (Style.FontDescription.Size * size / 100);
			list.CollapseAll ();
			list.ExpandAll ();
		}
		
		
		/// <summary>Returns selected entries id numbers.</summary>
		public override ArrayList GetSelectedEntries ()
		{
			Gtk.TreePath [] paths    = list.Selection.GetSelectedRows ();
			ArrayList       AllItems = new ArrayList ();
			Gtk.TreeStore   store    = list.Model as Gtk.TreeStore;
			
			int MainIter = -1;
			foreach (Gtk.TreePath pth in paths)
				if (pth.Depth == 1) {
					MainIter = pth.Indices [0];
					AllItems.AddRange (GetIterChildren (pth));
				} else {
					if (pth.Indices [0] != MainIter) {
						Gtk.TreeIter iter;
						store.GetIter (out iter, pth);
						AllItems.Add (store.GetValue (iter, (int) ColsEnum.ID));
					}
				}
			
			return AllItems;
		}
		
		
		/// <summary>Gets children (entries ids) of speciefied path.</summary>
		private ArrayList GetIterChildren (Gtk.TreePath pth)
		{
			Gtk.TreeIter  MainIter;
			Gtk.TreeIter  itr;
			Gtk.TreeStore store = list.Model as TreeStore;
			ArrayList     paths = new ArrayList ();
			
			store.GetIter (out MainIter, pth);
			for (bool tmp = store.IterChildren (out itr, MainIter); tmp == true; tmp = store.IterNext (ref itr))
				paths.Add (store.GetValue (itr, (int) ColsEnum.ID));
			
			return paths;
		}
		
		
		public override int GetDroppedEntryID (Gtk.DragDataReceivedArgs args)
		{
			Gtk.TreePath path;
			Gtk.TreeViewDropPosition pos;
			Gtk.TreeIter iter;
			
			list.GetDestRowAtPos (args.X, args.Y, out path, out pos);
			(list.Model as TreeStore).GetIter (out iter, path);
			
			if ((list.Model as TreeStore).IterHasChild (iter))
				return -1;
			
			return (int) (list.Model as TreeStore).GetValue (iter, (int) ColsEnum.ID);
		}
		
		
		/// <summary>Append a new entry.</summary>
		public override void AddEntry (Engine.EntryStruct entry, bool Expand)
		{
			string cmnt = Engine.Text.PrepareText (Engine.Text.Crop (entry.comment), true);
			if (cmnt != "")
				cmnt = cmnt + "\n";
			
			
			string tags = "";
			foreach (string str in entry.tags)
				if (tags.Length > 0)
					tags += ", " + str;
				else
					tags += str;
			
			if (tags != "")
				tags = "<b>" + Catalog.GetString ("Tags") + ":</b> " + Engine.Text.PrepareText (tags, false);

			
			string item = String.Format ("<span  weight=\"bold\">{0}</span>\n<span color=\"#555555\">{1}<span size=\"small\"><b>" +
							Catalog.GetString ("Date") + ":</b> {2}   <b>" + 
							Catalog.GetString ("Category") + ":</b> {3}   {4}</span></span>",
							Engine.Text.PrepareText (Engine.Text.Crop (DataTypes.DataTypes.GetDataType (entry.type).GetTitle (entry)), true),
				            cmnt, entry.GetDate (Engine.Settings.GetBool ("DateFirstYear")),
							Engine.Text.PrepareText (DataTypes.DataTypes.GetDataType (entry.type).Title, true), tags);
			
			Gtk.TreeIter MainIter = TreeIter.Zero;
			
			switch (SortBy) {
				case SortByEnum.Category:
					string title = DataTypes.DataTypes.GetDataType (entry.type).Title;
					MainIter = GetMainIter (title, ColsEnum.Category, title);
				break;
				
				case SortByEnum.Date:
					MainIter = GetMainIter (entry.GetDate (Engine.Settings.GetBool ("DateFirstYear")), ColsEnum.Date, entry.GetDate (true));
				break;
				
				case SortByEnum.Rating:
					if (entry.rating != 0)
						MainIter = GetMainIter (Catalog.GetString ("Rating") + " " + entry.rating.ToString (), ColsEnum.Rating, entry.rating);
					else
						MainIter = GetMainIter (Catalog.GetString ("No rating"), ColsEnum.Rating, entry.rating);
				break;
			}
			
			Gdk.Pixbuf RatingPixmap = EmptyP;
			switch (entry.rating) {
				case 1: RatingPixmap = Stars1; break;
				case 2: RatingPixmap = Stars2; break;
				case 3: RatingPixmap = Stars3; break;
			}
			
			
 			Gtk.TreeIter NewIter = (list.Model as TreeStore).AppendValues (MainIter, new object [] {item,
 					RatingPixmap, entry.id, DataTypes.DataTypes.GetDataType (entry.type).Title,
 					entry.date.ToString ("yyyy-MM-dd"), entry.rating});
 			
 			if (Expand == true) {
 				Gtk.TreeIter parent;
 				(list.Model as Gtk.TreeStore).IterParent (out parent, NewIter);
 				list.ExpandRow ((list.Model as TreeStore).GetPath (parent), false);
 			}
		}
		
		
		/// <summary>Gets main iter. If it doesn't exist, function creates a new main iter.</summary>
		private Gtk.TreeIter GetMainIter (string title, ColsEnum SortColumn, object SortData)
		{
			title = "<b>" + title + "</b>";
			Gtk.TreeStore store = list.Model as Gtk.TreeStore;
			Gtk.TreeIter itr2;
			
			for (bool tmp = store.GetIterFirst (out itr2); tmp == true; tmp = store.IterNext (ref itr2))
				if ((string) store.GetValue (itr2, 0) == title)
					return itr2;
			
			Gtk.TreeIter itr = (list.Model as TreeStore).AppendValues (new object [] {title});
			store.SetValue (itr, (int) SortColumn, SortData);
			
			return itr;
		}
		
		
		/// <summary>Gets entry iter by specified id.</summary>
		private Gtk.TreeIter GetIterByEntryID (int EntryID)
		{
			Gtk.TreeStore store = list.Model as Gtk.TreeStore;
			Gtk.TreeIter  itr, itr2;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (store.IterHasChild (itr))
					for (bool tmp2 = store.IterChildren (out itr2, itr); tmp2 == true; tmp2 = store.IterNext (ref itr2)) 
						if ((int) store.GetValue (itr2, (int) ColsEnum.ID) == EntryID)
							return itr2;
			
			return Gtk.TreeIter.Zero;
		}
		
		
		/// <summary>Modifies existing entry.</summary> 
		public override void ModifyEntry (Engine.EntryStruct NewEntry)
		{
			Gtk.TreeIter itr = GetIterByEntryID (NewEntry.id);
			if ((list.Model as TreeStore).IterIsValid (itr) == false)
				return;
			
			string cmnt = Engine.Text.PrepareText (Engine.Text.Crop (NewEntry.comment), true);
			if (cmnt != "")
				cmnt = cmnt + "\n";
			
			
			string tags = NewEntry.GetTagsString ();
			if (tags != "")
				tags = "<b>" + Catalog.GetString ("Tags") + ":</b> " + Engine.Text.PrepareText (tags, false);

			
			string item = String.Format ("<span  weight=\"bold\">{0}</span>\n<span color=\"#555555\">{1}<span size=\"small\"><b>" +
							Catalog.GetString ("Date") + ":</b> {2}  <b>" + 
							Catalog.GetString ("Category") + ":</b> {3}   {4}</span></span>",
							Engine.Text.PrepareText (Engine.Text.Crop (DataTypes.DataTypes.GetDataType (NewEntry.type).GetTitle (NewEntry)), true),
				            cmnt, NewEntry.GetDate (Engine.Settings.GetBool ("DateFirstYear")),
							Engine.Text.PrepareText (DataTypes.DataTypes.GetDataType (NewEntry.type).Title, true), tags);
			
			if (SortBy == SortByEnum.Rating) {
				Gtk.TreeIter parent;
				(list.Model as TreeStore).IterParent (out parent, itr);
				
				(list.Model as TreeStore).Remove (ref itr);
				if ((list.Model as TreeStore).IterNChildren (parent) == 0)
					(list.Model as TreeStore).Remove (ref parent);
				
				AddEntry (NewEntry, false);
				
				list.ExpandAll ();
			} else {
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.Item, item);
				
				Gdk.Pixbuf RatingPixmap = EmptyP;
				switch (NewEntry.rating) {
					case 1: RatingPixmap = Stars1; break;
					case 2: RatingPixmap = Stars2; break;
					case 3: RatingPixmap = Stars3; break;
				}
				
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.Stars, RatingPixmap);
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.ID, NewEntry.id);
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.Category, DataTypes.DataTypes.GetDataType (NewEntry.type).Title);
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.Date, NewEntry.date.ToString ());
				(list.Model as TreeStore).SetValue (itr, (int) ColsEnum.Rating, NewEntry.rating);
			}
		}
		
		
		/// <summary>Removes selected entry.</summary>
		public override void RemoveEntry (Engine.EntryStruct entry)
		{
			Gtk.TreeIter itr = GetIterByEntryID (entry.id);
			if ((list.Model as TreeStore).IterIsValid (itr) == false)
				return;
			
			Gtk.TreeIter parent;
			(list.Model as TreeStore).IterParent (out parent, itr);
			
			(list.Model as TreeStore).Remove (ref itr);
			if ((list.Model as TreeStore).IterNChildren (parent) == 0)
				(list.Model as TreeStore).Remove (ref parent);
		}
		
		
		public override bool Exists (int EntryID)
		{
			Gtk.TreeStore store = list.Model as Gtk.TreeStore;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (EntryID == (int) store.GetValue (itr, (int) ColsEnum.ID))
					return true;
			
			return false;
		}
	}
}
