/*
 *  GUI/list/list.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using Gdk;
using Incollector.GUI;
using Incollector.Engine;


namespace Incollector.GUI {

	public class TagsList : Gtk.VBox {
		
		public enum SortByEnum {
			Count  = 0,
			Active = 1,
			Name   = 2
		};
		
		private enum ColsEnum {
			Active    = 0,
			TagText   = 1,
			CountText = 2,
			Count     = 3
		};
		
		
		private TargetEntry [] TargetTable = new TargetEntry [] {
			new TargetEntry ("STRING", 0, 0)
		};
		
		
		private TreeView   list;
		private SortByEnum SortByValue;
		private bool       ShowingTagsListMenu;
		
		
		public SortByEnum SortBy {
			get {
				return SortByValue;
			}
			
			set {
				SortByValue = value;
				
				switch (value) {
					case SortByEnum.Active:
						(list.Model as ListStore).SetSortFunc (0, CompareActive);
						(list.Model as ListStore).SetSortColumnId (0, SortType.Descending);
					break;
					case SortByEnum.Count:
						(list.Model as ListStore).SetSortFunc (1, CompareCount);
						(list.Model as ListStore).SetSortColumnId (1, SortType.Descending);
					break;
					case SortByEnum.Name:
						(list.Model as ListStore).SetSortFunc (2, CompareName);
						(list.Model as ListStore).SetSortColumnId (2, SortType.Ascending);
					break;
				}
			}
		}
		
		
		public int EnabledTags {
			get {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				int enabled = 0;
				for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
					if ((bool) store.GetValue (itr, (int) ColsEnum.Active) == true)
						enabled++;
				
				return enabled;
			}
		}
		
		
		public int DisabledTags {
			get {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				int disabled = 0;
				for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
					if ((bool) store.GetValue (itr, (int) ColsEnum.Active) == false)
						disabled--;
				
				return disabled;
			}
		}
		
		
		public string [] Tags {
			get {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				string []     tags  = new string [(list.Model as ListStore).IterNChildren ()];
				int           i     = 0;
				
				for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
					tags [i] = (string) store.GetValue (itr, (int) ColsEnum.TagText);
					i++;
				}
				
				return tags;
			}
		}
		
		
		public delegate void TagActivityChangedHandler ();
		public delegate void RemoveTagActivateHandler  (string tag);
		public delegate void RenameTagActivateHandler  (string tag);
		
		
		public TagActivityChangedHandler TagsActivityChanged;
		public RemoveTagActivateHandler  RemoveTagActivate;
		public RenameTagActivateHandler  RenameTagActivate;
		
		
		/// <summary>Creates list view widget.</summary>
		public TagsList ()
		{
			// creating treeview widget
			list = new Gtk.TreeView ();
			
			Gtk.ListStore store = new Gtk.ListStore (typeof (bool), typeof (string), typeof (string), typeof (int));
			list.Model = store;
			list.EnableSearch = true;
			list.SearchColumn = (int) ColsEnum.TagText;
			list.HeadersVisible = false;
			list.DragDataGet += new DragDataGetHandler (on_drag_data_get);
			list.DragBegin   += new DragBeginHandler (on_drag_begin);
			list.ButtonReleaseEvent += new ButtonReleaseEventHandler (on_button_release_event);
			list.RowActivated += new RowActivatedHandler (on_row_activated);
			
			CellRendererToggle tgl = new CellRendererToggle ();
			tgl.Toggled += on_tag_toggled;
			
			Gtk.Drag.SourceSet (list, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
				    TargetTable, DragAction.Copy | DragAction.Move);
			
			list.AppendColumn ("", tgl, "active", 0);
			
			CellRendererText txtrend = new CellRendererText ();
			txtrend.Ellipsize = Pango.EllipsizeMode.End;
			TreeViewColumn col = new TreeViewColumn ("", txtrend, "markup", 1);
			col.Expand = true;
			list.AppendColumn (col);
			CellRendererText rend = new CellRendererText ();
			rend.Xalign = 1;
			list.AppendColumn ("", rend, "markup", 2);
			
			
			// creating scrollbar widget
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.Add (list);
			
			string srt = Engine.Settings.Get ("SortTags");
			if (srt == "counter")
				SortBy = SortByEnum.Count;
			else if (srt == "visible")
				SortBy = SortByEnum.Active;
			else
				SortBy = SortByEnum.Name;
			
			this.Add (scroll);
			this.ShowAll ();
		}
		
		
		protected void on_tag_toggled (object o, Gtk.ToggledArgs args)
		{
			Gtk.TreeIter  iter;
			Gtk.ListStore store = list.Model as ListStore;
		    if (store.GetIter (out iter, new Gtk.TreePath (args.Path))) {
				bool val = (bool) store.GetValue (iter, 0);
				store.SetValue (iter, 0, !val);
		    }
			
		    TagsActivityChanged ();
		}
		
		
		protected void on_drag_data_get (object o, DragDataGetArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr))
				args.SelectionData.Text = "incollector:tag:" + (list.Model as ListStore).GetPath (itr);
			else
				args.SelectionData.Text = "";
		}
		
		
		protected void on_drag_begin (object o, DragBeginArgs args)
		{
			Gtk.Drag.SetIconPixbuf (args.Context, new Gdk.Pixbuf (Assembly.GetEntryAssembly (), Icons.ADD_TAG_DND), 0, 0);
		}
		
		
		protected void on_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if (args.Event.Button == 3) {
				Glade.XML m = Glade.XML.FromAssembly ("gui.glade", "TagsListMenu", null);
				m.Autoconnect (this);
				Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("TagsListMenu");
				
				Gtk.RadioMenuItem counter, name, visible;
				counter = m.GetWidget ("counter") as Gtk.RadioMenuItem;
				name    = m.GetWidget ("name")    as Gtk.RadioMenuItem;
				visible = m.GetWidget ("visible") as Gtk.RadioMenuItem;
				
				ShowingTagsListMenu = true;
				
				if (SortBy == SortByEnum.Name)
					name.Active = true;
				else if (SortBy == SortByEnum.Active)
					visible.Active = true;
				else
					counter.Active = true;
				
				ShowingTagsListMenu = false;
				
				menu.ShowAll ();
				menu.Popup ();
			}
		}
		
		
		protected void on_counter_activate (object o, EventArgs args)
		{
			if (ShowingTagsListMenu)
				return;
			
			SortBy = SortByEnum.Count;
			Engine.Settings.Set ("SortTags", "counter");
		}
		
		
		protected void on_name_activate (object o, EventArgs args)
		{
			if (ShowingTagsListMenu)
				return;
			
			SortBy = SortByEnum.Name;
			Engine.Settings.Set ("SortTags", "name");
		}
		
		
		protected void on_visible_activate (object o, EventArgs args)
		{
			if (ShowingTagsListMenu)
				return;
			
			SortBy = SortByEnum.Active;
			Engine.Settings.Set ("SortTags", "visible");
		}	
		
		
		protected void on_show_only_this_tag_activate (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = list.Model as ListStore;
				Gtk.TreeIter  iter;
				
				string selected = (string) store.GetValue (itr, (int) ColsEnum.TagText);
				
				for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter))
					if (String.Compare ((string) store.GetValue (iter, (int) ColsEnum.TagText), selected) == 0)
						store.SetValue (iter, (int) ColsEnum.Active, true);
					else
						store.SetValue (iter, (int) ColsEnum.Active, false);
				
				TagsActivityChanged ();
			}
		}
		
		
		protected void on_show_all_tags_activate (object o, EventArgs args)
		{
			Gtk.ListStore store = list.Model as ListStore;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, (int) ColsEnum.Active, true);
			
			TagsActivityChanged ();
		}
		
		
		protected void on_rename_tag_activate (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr))
				if (RenameTagActivate != null)
					RenameTagActivate ((string) (mod as ListStore).GetValue (itr, (int) ColsEnum.TagText));
		}
		
		
		protected void on_remove_tag_activate (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr))
				if (RemoveTagActivate != null)
					RemoveTagActivate ((string) (mod as ListStore).GetValue (itr, (int) ColsEnum.TagText));
		}
		
		
		public string GetDraggingTag (Gtk.DragDataReceivedArgs args)
		{
			string TextPath = args.SelectionData.Text;
			try {
				TextPath = TextPath.Replace ("incollector:tag:", "");
				
				Gtk.TreeIter itr;
				(list.Model as ListStore).GetIter (out itr, new Gtk.TreePath (TextPath));
			
				return Engine.Text.PrepareText2 ((string) (list.Model as ListStore).GetValue (itr, (int) ColsEnum.TagText));
			} catch {
				return "";
			}
		}
		
		
		protected void on_row_activated (object o, RowActivatedArgs args)
		{	
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr))
				if ((EnabledTags == 1) && ((bool) (mod as ListStore).GetValue (itr, (int) ColsEnum.Active) == true))
					on_show_all_tags_activate (null, null);
				else
					on_show_only_this_tag_activate (null, null);
		}
		
		
		/// <summary>Compares 2 tags by active.</summary>
		private int CompareActive (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			return ((bool) store.GetValue (iter1, (int) ColsEnum.Active)).CompareTo
							((bool) store.GetValue (iter2, (int) ColsEnum.Active));
		}
		
		
		/// <summary>Compares 2 tags by their count.</summary>
		private int CompareCount (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			return Decimal.Compare ((int) store.GetValue (iter1, (int) ColsEnum.Count),
								(int) store.GetValue (iter2, (int) ColsEnum.Count));
		}
		
		
		/// <summary>Compares 2 tags by their name.</summary>
		private int CompareName (TreeModel store, TreeIter iter1, TreeIter iter2)
		{
			return String.Compare ((string) store.GetValue (iter1, (int) ColsEnum.TagText),
								(string) store.GetValue (iter2, (int) ColsEnum.TagText));
		}
		
		
		/// <summary>Clears tags list.</summary>
		public void Clear ()
		{
			(list.Model as ListStore).Clear ();
		}
		
		
		/// <summary>Adds a new tag.</summary>
		public void AddTag (string tag)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (String.Compare (Engine.Text.PrepareText2 ((string) store.GetValue (itr, (int) ColsEnum.TagText)), tag) == 0) {
					store.SetValue (itr, (int) ColsEnum.Count, (int) store.GetValue (itr, (int) ColsEnum.Count) + 1);
					store.SetValue (itr, (int) ColsEnum.CountText, "<b>" + (int) store.GetValue (itr, (int) ColsEnum.Count) + "</b>");
					return;
				}
			
			store.AppendValues (new object [] {true, Engine.Text.PrepareText (tag, false), "<b>1</b>", 1});
		}
		
		
		/// <summary>Adds tags from specyfied array list.</summary>
		public void AddTags (ArrayList tags)
		{
			foreach (string tag in tags)
				AddTag (tag);
		}
		
		
		/// <summary>Removes existing tag.</summary>
		public void RemoveTag (string tag)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (String.Compare (Engine.Text.PrepareText2 ((string) store.GetValue (itr, (int) ColsEnum.TagText)), tag) == 0) {
					if (((int) store.GetValue (itr, (int) ColsEnum.Count) - 1) == 0) {
						store.Remove (ref itr);
						return;
					}
					
					store.SetValue (itr, (int) ColsEnum.Count, (int) store.GetValue (itr, (int) ColsEnum.Count) - 1);
					store.SetValue (itr, (int) ColsEnum.CountText, "<b>" + (int) store.GetValue (itr, (int) ColsEnum.Count) + "</b>");
					return;
				}
		}
		
		
		/// <summary>Removes tags from specyfied array list.</summary>
		public void RemoveTags (ArrayList tags)
		{
			foreach (string tag in tags)
				RemoveTag (tag);
		}
		
		
		/// <summary>Looks for diifs between tags1 and tags2. Adds and removes deleted entries.</summary>
		public void ApplyDiff (ArrayList tags1, ArrayList tags2)
		{
			int cnt = tags1.Count, cnt2 = tags2.Count;
			
			for (int i = 0; i < cnt; i++)
				for (int j = 0; j < cnt2; j++) {
					if (String.Compare ((string) tags1 [i], (string) tags2 [j]) == 0) {
						tags1.RemoveAt (i);
						tags2.RemoveAt (j);
						
						cnt--;
						i--;
						cnt2--;
						j--;
						
						break;
					}
				}
			
			AddTags (tags2);
			RemoveTags (tags1);
		}
		
		
		/// <summary>Returns enabled tags list.</summary>
		public ArrayList EnabledTagsList ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			ArrayList    lst    = new ArrayList ();
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if ((bool) store.GetValue (itr, (int) ColsEnum.Active) == true)
					lst.Add (Engine.Text.PrepareText2 ((string) store.GetValue (itr, (int) ColsEnum.TagText)));
			
			return lst;
		}
		
		
		/// <summary>Returns disabled tags list.</summary>
		public ArrayList DisabledTagsList ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			ArrayList    lst    = new ArrayList ();
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if ((bool) store.GetValue (itr, (int) ColsEnum.Active) == false)
					lst.Add (Engine.Text.PrepareText2 ((string) store.GetValue (itr, (int) ColsEnum.TagText)));
			
			return lst;
		}
		
		
		/// <summary>Enables all the tags.</summary>
		public void EnableAllTags ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, (int) ColsEnum.Active, true);
			
			TagsActivityChanged ();
		}
		
		
		/// <summary>Disables all the tags.</summary>
		public void DisableAllTags ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, (int) ColsEnum.Active, false);
			
			TagsActivityChanged ();
		}
		
		
		/// <summary>Enables speciefied tag and disables others.</summary>
		public void EnableOneTag (string tag)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (tag == (string) store.GetValue (itr, (int) ColsEnum.TagText))
					store.SetValue (itr, (int) ColsEnum.Active, true);
				else
					store.SetValue (itr, (int) ColsEnum.Active, false);
			
			TagsActivityChanged ();
		}
		

		/// <summary>All tags which are in DisabledTags list will be disabled. Others will be enabled.</summary>		
		public void LoadTagsActive (ArrayList DisabledTags)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				foreach (string tag in DisabledTags)
					if (tag == (string) store.GetValue (itr, (int) ColsEnum.TagText)) {
						store.SetValue (itr, (int) ColsEnum.Active, false);
						DisabledTags.Remove (tag);
						goto NextStep;
					}
				
				store.SetValue (itr, (int) ColsEnum.Active, true);
				
				NextStep:
					;
			}
			
			
			TagsActivityChanged ();
		}
	}
}
