/*
 *  Plugins/Plugins.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using System.Reflection;
using System.Xml;
using System.IO;
using Mono.Unix;
using Incollector.Engine;
using Gtk;


namespace Incollector.Plugins {

	public enum InstallationResult {
		Successful    = 0,
		PluginUpdated = 1,
		AlreadyExists = 2,
		OlderVersion  = 3,
		Dependences   = 4
	}


	/*
	 * Plugin class
	 */
	 	
	public static class Plugins {
	
		public struct PluginStruct {
			public string  FileName;
			public Version Ver;
			
			public PluginStruct (string file, Version version)
			{
				this.FileName = file;
				this.Ver      = version;
			}
		}
		
		public static ArrayList PluginsList = new ArrayList ();
		public static ArrayList AllPlugins  = new ArrayList ();
		
		
		/* Loads all plugins from user and global plugins directory */ 
		public static void LoadAllPlugins ()
		{
			GetPluginsList (".");
			
			if (Directory.Exists (Engine.FilesPaths.GlobalPluginsDirectory))
				GetPluginsList (Engine.FilesPaths.GlobalPluginsDirectory);
			
			if (Directory.Exists (Engine.FilesPaths.UserPluginsDirectory))
				GetPluginsList (Engine.FilesPaths.UserPluginsDirectory);
			
			
			for (int i = 0; i < AllPlugins.Count; i++) {
				PluginStruct plug = (PluginStruct) AllPlugins [i];
				LoadPlugin (plug.FileName);
			}
		}
		
		
		/* Returns plugins list */
		private static void GetPluginsList (string directory)
		{
			FileInfo [] pgns = (new DirectoryInfo (directory)).GetFiles ("*.dll");
			if (pgns.Length == 0)
				return;
			
			foreach (FileInfo pgn in pgns) {
				Version version = new PluginInfo (pgn.FullName.Replace (pgn.Extension, ".xml")).version;
				for (int i = 0; i < AllPlugins.Count; i++) {
					PluginStruct plugin = (PluginStruct) AllPlugins [i];
					
					if (Path.GetFileName (plugin.FileName) == Path.GetFileName (pgn.FullName)) {
						if (version > plugin.Ver) {
							plugin.FileName = pgn.FullName;
							plugin.Ver    = version;
							AllPlugins [i] = plugin;
						}
						
						goto next;
					}
				}
				
				AllPlugins.Add (new PluginStruct (pgn.FullName, version));
				
				next: ;
			}
		}
		
		
		/* Loads plugin */
		public static void LoadPlugin (string PluginFile)
		{
			try {
				Assembly asm = Assembly.LoadFrom (PluginFile);
				foreach (Type type in asm.GetTypes ()) {
					if (type.IsSubclassOf (typeof (Plugin))) {
						// loading plugin
						Plugin plugin;
						
						try {
							plugin = (Plugin) Activator.CreateInstance (type);
							plugin.Init ();
						} catch (System.Exception exc) {
							GUI.PluginErrorDialog dlg = new GUI.PluginErrorDialog (
								Path.GetFileName (PluginFile), PluginFile, exc.StackTrace);
							dlg.dialog.Run ();
							dlg.dialog.Destroy ();
							
							continue;
						}
						
						plugin.Path = PluginFile;
						PluginsList.Add (plugin);
					}
				}
			} catch (System.Exception msg) {
				Console.WriteLine (Catalog.GetString ("{0} plugin couldn't be loaded !"), Path.GetFileName (PluginFile));
			}
		}
		
		
		public static InstallationResult InstallPlugin (string FileName, out ArrayList Dependences)
		{
			// unpacking
			System.Random rand = new System.Random ();
			string UnpackDirectory = Path.Combine (FilesPaths.UserPluginsDirectory, Path.GetFileName (FileName) + ".tmp." + rand.Next (1000, 100000).ToString ());
			Incpkg.Unpack (FileName, UnpackDirectory);
			
			// checking version
			string PluginFileName = Path.GetFileName (FileName);
			PluginFileName = PluginFileName.Replace (Path.GetExtension (PluginFileName), "");
			string FullPluginFileName = FilesPaths.UserPluginsDirectory + FilesPaths.Separator + PluginFileName;
			
			
			PluginInfo info = new PluginInfo (Path.Combine (UnpackDirectory, PluginFileName + ".xml"));
			Version version = info.version;
			
			// checking dependences
			ArrayList deps = info.CheckDependences ();
			if (deps.Count > 0) {
				Dependences = deps;
				Directory.Delete (UnpackDirectory, true);
				return InstallationResult.Dependences;
			} else
				Dependences = new ArrayList ();
			
			
			foreach (PluginStruct plugin in AllPlugins) {
				if (string.Compare (Path.GetFileName (plugin.FileName), PluginFileName + ".dll") == 0) {
					if (version < plugin.Ver) { // newer version exists
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.OlderVersion;
					} else if (version == plugin.Ver) { // the same version already exists
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.AlreadyExists;
					} else { // older version exists -> updating !
						if (plugin.FileName.StartsWith (FilesPaths.UserPluginsDirectory) == true) { // if not user plugins dir
							try {
								Directory.Delete (FullPluginFileName, true);
							} catch {}
							
							try {
								File.Delete (FullPluginFileName + ".dll");
							} catch {}
							
							try {
								File.Delete (FullPluginFileName + ".xml");
							} catch {}
						}
						
						try {
							System.IO.File.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName + ".dll", FullPluginFileName + ".dll");
						} catch {}
						
						try {
							System.IO.File.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName + ".xml", FullPluginFileName + ".xml");
						} catch {}
						
						try {
							System.IO.Directory.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName, FullPluginFileName);
						} catch {}
						
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.PluginUpdated;
					}
				}
			}
			
			
			try {
				System.IO.File.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName + ".dll", FullPluginFileName + ".dll");
			} catch {}
			
			try {
				System.IO.File.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName + ".xml", FullPluginFileName + ".xml");
			} catch {}
			
			try {
				System.IO.Directory.Move (UnpackDirectory + FilesPaths.Separator + PluginFileName, FullPluginFileName);
			} catch {}
			
			Directory.Delete (UnpackDirectory, true);
			return InstallationResult.Successful;
		}
		
		
		public static bool UninstallPlugin (string PluginPath)
		{
			string FullPluginFileName = PluginPath.Remove (PluginPath.Length - 
				Path.GetExtension (PluginPath).Length, Path.GetExtension (PluginPath).Length);
			
			if (FullPluginFileName.StartsWith (FilesPaths.UserPluginsDirectory) == false)
				return false;
			
			
			try {
				Directory.Delete (FullPluginFileName, true);
			} catch {}
			
			try {
				File.Delete (FullPluginFileName + ".dll");
			} catch {}
			
			try {
				File.Delete (FullPluginFileName + ".xml");
			} catch {}
			
			return true;
		}
	}
}
