/*
 *  Types/DataType.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using System.Xml;
using Gtk;


namespace Incollector.DataTypes {
	
	/// <summary>Managing data types</summary>
	public class DataTypes {
		static public ArrayList DataTypesList;
		
		
		/// <summary>Loads all the data types.</summary>
		static public void Create ()
		{
			DataTypesList = new ArrayList ();
			
			DataTypesList.Add (new Conversation ());
			DataTypesList.Add (new Note ());
			DataTypesList.Add (new Quote ());
			DataTypesList.Add (new SerialNumber ());
			DataTypesList.Add (new SourceCode ());
			DataTypesList.Add (new WebAddress ());
			DataTypesList.Add (new Word ());
		}
		
		
		/// <summary>Returns data type by name.</summary>
		/// <param name="name">Name of data type</param>
		static public DataType GetDataType (string name)
		{
			foreach (DataType tpe in DataTypesList)
				if (tpe.Name == name)
					return tpe;
			
			return null;
		}
	}



	/// <summary>Describes a data type.</summary>
	public abstract class DataType {

		/// <summary>Returns name of data type</summary>
		/// <remarks>Name is short identifier of data type for DataTypesList. It is not displayed.</remarks>
		public abstract string Name {
			get ;
		}
		
		/// <summary>Returns title of data type</summary>
		/// <remarks>Title is full name of data type. It is used in GUI.</remarks>
		public abstract string Title {
			get ;
		}
		
		
		/// <summary>Returns data type menu item text. For example: Add a note.</summary>
		public abstract string MenuText {
			get ;
		}
		
		
		/// <summary>Returns a widget that can be focused default.</summary>
		public abstract Gtk.Widget DefaultFocusWidget {
			get ;
		}
		
		
		/// <summary>Returns a widget that can be focused.</summary>
		public abstract Gtk.Widget FocusWidget {
			get ;
		}
		
		
		public DataType ()
		{
		}
		
		
		/// <summary>Reads data from XML</summary>
		/// <returns>Entry with filled "data" field</returns>
		/// <param name="reader">XmlTextReader</param>
		/// <param name="entry">An entry without "data" field filled.</param>
		/// <remarks>This XML code to save in entries database.</remarks>
		public abstract Engine.EntryStruct ReadXml (XmlTextReader reader, Engine.EntryStruct entry);
		
		/// <summary>Writes data to XML</summary>
		/// <param name="writer">XmlTextWriter</param>
		/// <param name="data">Data which are converted to XML</param>
		/// <remarks>This XML code to save in entries database.</remarks>
		public abstract void WriteXml (XmlTextWriter writer, Engine.EntryStruct data);
		
		/// <summary>Returns widget to edit entry</summary>
		public Gtk.Widget EntryEditWidget ()
		{
			return EntryEditWidget (new Engine.EntryStruct ());
		}
		
		/// <summary>Returns widget to edit entry with filled one widget.</summary>
		public Gtk.Widget EntryEditWidget (string content)
		{
			return EntryEditWidget (new Engine.EntryStruct (), content);
		}
		
		/// <summary>Returns widget to edit entry</summary>
		/// <param name="entry">Widget is filled by this entry</param>
		public abstract Gtk.Widget EntryEditWidget (Engine.EntryStruct entry);
		
		/// <summary>Returns widget to edit entry.</summary>
		/// <param name="entry">Widget is filled by this entry</param>
		/// <param name="content">The content of the one filled widget.</param>
		public abstract Gtk.Widget EntryEditWidget (Engine.EntryStruct entry, string content);
		
		/// <summary>Add "changed" EntryEditWidget event handler</summary>
		public abstract void AddChangedEvent (EventHandler hdlr);

		/// <summary>Returns widget to preview entry</summary>
		public abstract Gtk.Widget EntryPreviewWidget (Engine.EntryStruct entry);
		
		/// <summary>Returns title of entry</summary>
		/// <remarks>Displayed on entries list (GUI)</remarks>
		public abstract string GetTitle (Engine.EntryStruct entry);
		
		/// <summary>Returns data from edit widget.</summary>
		public abstract object GetData ();
		
		/// <summary>Search for "query" in "entry"</summary>
		/// <returns>True, if data was found</returns>
		public abstract bool Search (Engine.EntryStruct entry, string query);
		
		/// <summary>Returns true, if Edit entry is filled.</summary>
		public abstract bool IsFormFilledIn ();
	}
}
